#! /usr/bin/python

"""    Multipeer, Copyright 2003 Andreas Bergstrm and Audun Vaaler

    This file is part of Multipeer.
    
    Multipeer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"""

import multipeer, time, os, threading, string, sha, mimetypes, filelib
from wxPython.wx import *

#IDs of the main event
ID_MAIN          = 100
ID_SEARCH        = 101
ID_DOWNLOAD      = 102
ID_ABORT         = 103
ID_EXIT          = 104
ID_ABOUT         = 105
ID_ADDFILE       = 106
ID_UNPUBLISH     = 107
ID_PEERS         = 108
ID_PEERS_CLOSE   = 109
ID_PEERS_REFRESH = 110
ID_INFO          = 111

TIMEOUT = 10 # Seconds

debug = 0

def ToReadableUnits(i):
    if i < 1000:
        return "%i B" % i
    elif i < 1000000:
        return "%.1f kB" % (i / pow(2.0, 10))
    elif i < 1000000000:
        return "%.1f MB" % (i / pow(2.0, 20))
    else:
        return "%.1f GB" % (i / pow(2.0, 30))

class ProgressTimer(wxTimer):
    def __init__(self, f):
        wxTimer.__init__(self)
        
        self.F = f

    def Notify(self):
        self.F()

class InfoFrame(wxFrame):
    def __init__(self, parent, hash, info):
        wxFrame.__init__(self, parent, -1, info[0], size = (200, 150))
        
        self.parent = parent
        self.hash   = hash
        self.info   = info

        self.lblName = wxStaticText(self, -1, "Name:")
        self.lblCat = wxStaticText(self, -1, "Category:")
        self.lblMIME = wxStaticText(self, -1, "MIME type:")
        self.lblSize = wxStaticText(self, -1, "Size:")
        self.lblDate = wxStaticText(self, -1, "Date:")
        self.lblHash = wxStaticText(self, -1, "Hash:")
        
        self.txtName = wxTextCtrl(self, -1, info[0])
        self.txtMIME = wxTextCtrl(self, -1, info[5])
        self.txtCat  = wxTextCtrl(self, -1, info[2])
        self.txtSize  = wxTextCtrl(self, -1, str(info[6]))
        self.txtDate  = wxTextCtrl(self, -1, info[1])
        self.txtHash  = wxTextCtrl(self, -1, hash)
        self.boxDesc = wxStaticBox(self, -1, label = "Description")
        self.sizerDesc = wxStaticBoxSizer(self.boxDesc, wxVERTICAL)
        self.txtDesc = wxTextCtrl(self, -1, info[4], style=wxTE_MULTILINE | wxHSCROLL)
        self.txtDesc.SetDimensions(-1, -1, -1, 80)

        self.butClose = wxButton(self, wxOK, "Close")

        self.sizerFrame   = wxBoxSizer(wxVERTICAL)

        self.sizerTop = wxFlexGridSizer(2, 2, 2, 16)
        self.sizerDesc.Add(self.txtDesc, 1, wxEXPAND | wxALL, 4)

        self.sizerTop.Add(self.lblName, 0, wxALL, 0)
        self.sizerTop.Add(self.txtName, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblMIME, 0, wxALL, 0)
        self.sizerTop.Add(self.txtMIME, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblCat, 0, wxALL, 0)
        self.sizerTop.Add(self.txtCat, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblSize, 0, wxALL, 0)
        self.sizerTop.Add(self.txtSize, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblDate, 0, wxALL, 0)
        self.sizerTop.Add(self.txtDate, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblHash, 0, wxALL, 0)
        self.sizerTop.Add(self.txtHash, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.AddGrowableCol(1)

        self.sizerFrame.Add(self.sizerTop, 0, wxALL | wxEXPAND, 8)
        self.sizerFrame.Add(self.sizerDesc, 1, wxALL | wxEXPAND, 4)
        self.sizerFrame.Add(self.butClose, 0, wxALL, 4)

        self.SetSizer(self.sizerFrame)
        self.sizerFrame.SetSizeHints(self)

        self.CentreOnParent(wxBOTH)

        EVT_CLOSE(self, self.OnClose)
        EVT_BUTTON(self, wxOK, self.OnClose)

    def OnClose(self, evt):
        del self.parent.infoFrames[self.hash]
        self.Destroy()

class PeersFrame(wxFrame):
    def __init__(self, parent):
        wxFrame.__init__(self, parent, -1, "Other peers", size = (200, 150))

        self.parent = parent
        self.prevPongs = []

        self.sizerFrame = wxBoxSizer(wxVERTICAL)

        self.listPeers = wxListBox(self, -1)
        self.boxPeers = wxStaticBox(self, -1, label = "Other peers on the network")
        self.sizerPeers = wxStaticBoxSizer(self.boxPeers, wxVERTICAL)
        self.sizerPeers.Add(self.listPeers, 1, wxALL | wxEXPAND, 4)

        self.sizerButtons = wxBoxSizer(wxHORIZONTAL)
        self.butClose = wxButton(self, ID_PEERS_CLOSE, "Close")
        self.butRefresh = wxButton(self, ID_PEERS_REFRESH, "Refresh")
        self.sizerButtons.Add(self.butClose, 0, wxALL, 4)
        self.sizerButtons.Add(self.butRefresh, 0, wxALL, 4)

        self.sizerFrame.Add(self.sizerPeers, 1, wxLEFT | wxEXPAND, 4)
        self.sizerFrame.Add(self.sizerButtons, 0, wxLEFT, 4)

        self.SetSizer(self.sizerFrame)
        self.sizerFrame.SetSizeHints(self)

        self.timerPeers = ProgressTimer(self.UpdatePeers)
        self.timerPeers.Start(100, false)

        self.parent.main.ping()

        EVT_CLOSE(self, self.KickBucket)
        EVT_BUTTON(self, ID_PEERS_CLOSE, self.KickBucket)
        EVT_BUTTON(self, ID_PEERS_REFRESH, self.Refresh)

    def Refresh(self, evt):
        self.listPeers.Clear()
        self.prevPongs = []
        self.parent.main.ping()

    def KickBucket(self, f): # wxWindows seems to get confused by more logical names...
        self.timerPeers.Stop()

        self.Destroy()

    def UpdatePeers(self):
        pongs = self.parent.main.getPongers().keys()

        if pongs != self.prevPongs:
            self.listPeers.Set(pongs)
            self.prevPongs = pongs

class MainFrame(wxFrame): # :-)
    def __init__(self, parent, id, title):
        wxFrame.__init__(self, parent, ID_MAIN, title, size = (500, 150),
                         style = wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE)

        self.searchMode = 0
        self.searchStartTime = 0

        self.infoFrames = {}

        self.timerSearchProgress = ProgressTimer(self.UpdateSearchProgress)
        self.timerDownloadProgress = ProgressTimer(self.UpdateDownloadProgress)
        self.timerDownloadProgress.Start(1000, false)

        # Menu
        self.filemenu = wxMenu()
        self.windowmenu = wxMenu()
        self.helpmenu = wxMenu()
        self.menubar = wxMenuBar()

        # File menu
        self.filemenu.Append(ID_ADDFILE, "Publish &File", "Publish (share) a file")
        self.filemenu.AppendSeparator()
        self.filemenu.Append(ID_EXIT, "E&xit", "Exit from program")
        self.menubar.Append(self.filemenu, "&File")

        # Window menu
        self.windowmenu.Append(ID_PEERS, "Show &peers", "Show other peers")
        self.menubar.Append(self.windowmenu, "Peers")

        # Help menu
        self.helpmenu.Append(ID_ABOUT, "&About", "More information about this program")
        self.menubar.Append(self.helpmenu, "&Help")

        self.SetMenuBar(self.menubar)

        # Tabs
        self.notebookFrame = wxNotebook(self, -1)
        self.panelD = wxPanel(self.notebookFrame, -1)
        self.panelP = wxPanel(self.notebookFrame, -1)

        self.notebookFrame.AddPage(self.panelD, "Downloads")
        self.notebookFrame.AddPage(self.panelP, "Published files")

        # Search row
        self.sizerSearch = wxBoxSizer(wxHORIZONTAL)

        self.textSearch  = wxTextCtrl(self.panelD, ID_SEARCH, "*", style = wxTE_PROCESS_ENTER)
        #self.sizerSearch.Add(self.textSearch, 1, wxALL, 4)

        self.buttonSearch = wxButton(self.panelD, ID_SEARCH, "Search")
        self.sizerSearch.Add(self.buttonSearch, 0, wxALL, 4)

        self.gaugeProgress = wxGauge(self.panelD, -1, 100)
        self.gaugeProgress.SetValue(0)

        # Results row
        self.boxResults = wxStaticBox(self.panelD, -1, label = "Search results")
        self.sizerResults = wxStaticBoxSizer(self.boxResults, wxVERTICAL)
        
        self.listResults = wxListCtrl(self.panelD, ID_DOWNLOAD, style = wxLC_REPORT | wxLC_VRULES | wxSUNKEN_BORDER)

        self.listResults.InsertColumn(0, "#")
        self.listResults.InsertColumn(1, "Name")
        self.listResults.InsertColumn(2, "Date")
        self.listResults.InsertColumn(3, "Category")
        self.listResults.InsertColumn(4, "Size")

        self.listResults.SetColumnWidth(0, wxLIST_AUTOSIZE_USEHEADER)
        self.listResults.SetColumnWidth(1, 160)
        self.listResults.SetColumnWidth(2, wxLIST_AUTOSIZE_USEHEADER)
        self.listResults.SetColumnWidth(3, wxLIST_AUTOSIZE_USEHEADER)
        self.listResults.SetColumnWidth(4, wxLIST_AUTOSIZE_USEHEADER)

        self.sizerResults.Add(self.listResults, 1, wxEXPAND | wxALL, 4)

        self.sizerDownloadInfo = wxBoxSizer(wxHORIZONTAL)
        self.buttonResults = wxButton(self.panelD, ID_DOWNLOAD, "Download")
        self.buttonInfo    = wxButton(self.panelD, ID_INFO, "More info...")
        self.sizerDownloadInfo.Add(self.buttonResults, 0, wxALL, 0)
        self.sizerDownloadInfo.Add(self.buttonInfo, 0, wxLEFT, 4)
        self.sizerResults.Add(self.sizerDownloadInfo, 0, wxALL, 4)

        # Downloads row
        self.boxDownloads = wxStaticBox(self.panelD, -1, label = "Downloads")
        self.sizerDownloads = wxStaticBoxSizer(self.boxDownloads, wxVERTICAL)
        
        #self.listDownloads = wxListBox(self.panelD, -1, style = wxLB_EXTENDED)
        self.listDownloads = wxListCtrl(self.panelD, -1, style = wxLC_REPORT | wxLC_VRULES | wxSUNKEN_BORDER)
        self.listDownloads.nameMapping = {}
        self.listDownloads.hashToInfoMap = {}
        self.listDownloads.numToHashMap  = {}

        self.listDownloads.InsertColumn(0, "#")
        self.listDownloads.InsertColumn(1, "Name")
        self.listDownloads.InsertColumn(2, "Status")

        self.listDownloads.SetColumnWidth(0, wxLIST_AUTOSIZE_USEHEADER)
        self.listDownloads.SetColumnWidth(1, 160)
        self.listDownloads.SetColumnWidth(2, wxLIST_AUTOSIZE_USEHEADER)

        self.buttonAbort = wxButton(self.panelD, ID_ABORT, "Abort download")

        self.sizerDownloads.Add(self.listDownloads, 1, wxEXPAND | wxALL, 4)
        self.sizerDownloads.Add(self.buttonAbort, 0, wxALL, 4)

        # Downloads page sizer
        self.sizerFrame = wxBoxSizer(wxVERTICAL)

        self.sizerFrame.Add(self.sizerSearch, 0, wxEXPAND | wxALL, 4)
        self.sizerFrame.Add(self.sizerResults, 1, wxEXPAND | wxALL, 4)
        self.sizerFrame.Add(self.sizerDownloads, 1, wxEXPAND | wxALL, 4)

        self.panelD.SetSizer(self.sizerFrame)
        #self.SetSizer(self.sizerFrame)
        self.sizerFrame.SetSizeHints(self)

        # Published files sizer
        self.sizerPublished = wxBoxSizer(wxVERTICAL)

        # Published files buttons sizer
        self.sizerPublishedButtons = wxBoxSizer(wxHORIZONTAL)

        # Published files list
        self.listPublished = wxListCtrl(self.panelP, ID_DOWNLOAD, style = wxLC_REPORT | wxLC_VRULES | wxSUNKEN_BORDER)

        self.listPublished.InsertColumn(0, "#")
        self.listPublished.InsertColumn(1, "Name")
        self.listPublished.InsertColumn(2, "Date")
        self.listPublished.InsertColumn(3, "Category")
        self.listPublished.InsertColumn(4, "Size")

        self.listPublished.SetColumnWidth(0, wxLIST_AUTOSIZE_USEHEADER)
        self.listPublished.SetColumnWidth(1, 160)
        self.listPublished.SetColumnWidth(2, wxLIST_AUTOSIZE_USEHEADER)
        self.listPublished.SetColumnWidth(3, wxLIST_AUTOSIZE_USEHEADER)
        self.listPublished.SetColumnWidth(4, wxLIST_AUTOSIZE_USEHEADER)

        self.sizerPublished.Add(self.listPublished, 1, wxEXPAND | wxALL, 4)

        # Published files buttons
        self.buttonPublish = wxButton(self.panelP, ID_ADDFILE, "Publish new file...")
        self.sizerPublishedButtons.Add(self.buttonPublish, 0, wxALL, 4)

        self.buttonUnpublish = wxButton(self.panelP, ID_UNPUBLISH, "Unpublish")
        self.sizerPublishedButtons.Add(self.buttonUnpublish, 0, wxALL, 4)

        # Published files: Tidying up
        self.sizerPublished.Add(self.sizerPublishedButtons, 0, wxALL, 0)

        self.panelP.SetSizer(self.sizerPublished)
        self.sizerPublished.SetSizeHints(self)

        # Peers window
        self.framePeers = None #PeersFrame(self)

        # Add eventhandlers
        EVT_CLOSE(self, self.MainEvent)
        EVT_BUTTON(self, ID_SEARCH, self.Search)
        EVT_BUTTON(self, ID_DOWNLOAD, self.Download)
        EVT_BUTTON(self, ID_INFO, self.ShowInfo)
        EVT_BUTTON(self, ID_ADDFILE, self.AddFile)
        EVT_BUTTON(self, ID_UNPUBLISH, self.Unpublish)
        EVT_BUTTON(self, ID_ABORT, self.AbortDownload)
        EVT_TEXT_ENTER(self, ID_SEARCH, self.Search)
        EVT_LIST_ITEM_ACTIVATED(self, ID_DOWNLOAD, self.Download)
        EVT_LIST_ITEM_SELECTED(self, ID_DOWNLOAD, self.UpdateSearchControls)
        EVT_MENU(self, ID_ABOUT, self.OnAbout)
        EVT_MENU(self, ID_EXIT, self.MainEvent)
        EVT_MENU(self, ID_ADDFILE, self.AddFile)
        EVT_MENU(self, ID_PEERS, self.ShowPeers)
        EVT_UPDATE_UI(self, ID_UNPUBLISH, self.UpdateUnpublishButton)
        EVT_UPDATE_UI(self, ID_ABORT, self.UpdateAbortButton)
        EVT_UPDATE_UI(self, ID_INFO, self.UpdateInfoButton)

        self.UpdateSearchControls()
        self.UpdatePublished()

        w, h = self.GetSizeTuple()

        w *= 2
        h *= 1.5

        self.SetDimensions(-1, -1, w, h)
        
        self.Show(true)

        self.main = multipeer.Main()

    def UpdateAbortButton(self, evt):
        if self.listDownloads.GetSelectedItemCount():
            evt.Enable(true)
        else:
            evt.Enable(false)

    def UpdateInfoButton(self, evt):
        if self.listResults.GetSelectedItemCount():
            evt.Enable(true)
        else:
            evt.Enable(false)

    def Unpublish(self, event):
        pos = self.listPublished.GetNextItem(-1, state = wxLIST_STATE_SELECTED)
        index = int(self.listPublished.GetItem(pos, 0).GetText()) - 1
        
        hash = self.listPublished.numToHashMap[index]

        self.main.removeFileFromSharing(hash)
        #del self.main.downloads[hash]
        self.UpdatePublished()

    def UpdateUnpublishButton(self, evt):
        if self.listPublished.GetSelectedItemCount():
            evt.Enable(true)
        else:
            evt.Enable(false)

    def ShowPeers(self, event):
        try:
            self.framePeers.Show()
        except (wxPyDeadObjectError, AttributeError):
            self.framePeers = PeersFrame(self)
            self.framePeers.Show()

    def ShowInfo(self, event):
        pos = self.listResults.GetNextItem(-1, state = wxLIST_STATE_SELECTED)
        index = int(self.listResults.GetItem(pos, 0).GetText()) - 1
        
        hash   = self.listResults.numToHashMap[index]
        info   = self.listResults.hashToInfoMap[hash]

        if not self.infoFrames.has_key(hash):
            self.infoFrames[hash] = InfoFrame(self, hash, info)
            
        self.infoFrames[hash].Show()

    def AbortDownload(self, event):
        pos = self.listResults.GetNextItem(-1, state = wxLIST_STATE_SELECTED)
        index = int(self.listResults.GetItem(pos, 0).GetText()) - 1
        
        hash = self.listResults.numToHashMap[index]

        self.main.stopDownload(hash)
        del self.main.downloads[hash]
        self.UpdateDownloadProgress(full = 1)

    def UpdateSearchControls(self, junk = None, junk2 = None):
        if self.searchMode:        
            self.sizerSearch.Remove(self.textSearch)
            self.textSearch.Show(false)
            self.sizerSearch.Prepend(self.gaugeProgress, 1, wxEXPAND | wxALL, 4)
            self.gaugeProgress.Show(true)
            self.sizerSearch.Layout()

            self.buttonSearch.SetLabel("Abort")
            self.buttonResults.Disable()
            self.listResults.Disable()
        else:
            self.buttonSearch.SetLabel("Search")
            
            self.gaugeProgress.Show(false)
            self.sizerSearch.Remove(self.gaugeProgress)
            self.sizerSearch.Remove(self.textSearch)
            self.textSearch.Show(true)
            self.sizerSearch.Prepend(self.textSearch, 1, wxEXPAND | wxALL, 4)
            self.sizerSearch.Layout()
            
            #if len(self.listResults.GetSelections()) > 0:
            if self.listResults.GetSelectedItemCount() > 0:
                self.buttonResults.Enable(true)
            else:
                self.buttonResults.Disable()
            self.listResults.Enable(true)

    #def UpdateDownloadControls(self):
    #    if self.downloadMode:
    #        self.buttonDownloads.Disable()
    #        self.listDownloads.Disable()
    #    else:
    #        self.buttonDownloads.Enable(true)
    #        self.listDownloads.Enable(true)

    def UpdateSearchProgress(self):
        v = min(int(((time.time() - self.searchStartTime) / TIMEOUT) * 100), 100)

        if v < 100:
            self.gaugeProgress.SetValue(v)
        else:
            self.StopSearching()

        results = self.main.getQueryResult(self.sessid)

        ans = results.getAnswer()

        changes = 0

        for item in ans:
            #name, date, cat, ccat, desc, mime, hash, length = item

            name   = item["Name"].strip()
            date   = item["Date"].strip()
            cat    = item["Category"].strip()
            ccat   = item["CCategory"].strip()
            desc   = item["Desc"].strip()
            mime   = item["MIME"].strip()
            hash   = item["HASH"].strip()
            length = int(item["length"].strip())

            if hash not in self.listResults.hashToInfoMap.keys():
                changes = 1
                
                index = self.listResults.InsertStringItem(self.listResults.GetItemCount(), "")

                self.listResults.hashToInfoMap[hash] = (name, date, cat, ccat, desc, mime, length)
                self.listResults.numToHashMap[index] = hash

                self.listResults.SetStringItem(index, 0, str(index + 1))
                self.listResults.SetStringItem(index, 1, name)
                self.listResults.SetStringItem(index, 2, date)
                self.listResults.SetStringItem(index, 3, cat)
                self.listResults.SetStringItem(index, 4, ToReadableUnits(length))

        if changes:
            self.listResults.SetColumnWidth(0, wxLIST_AUTOSIZE)
            self.listResults.SetColumnWidth(1, wxLIST_AUTOSIZE)
            self.listResults.SetColumnWidth(2, wxLIST_AUTOSIZE)
            self.listResults.SetColumnWidth(3, wxLIST_AUTOSIZE)
            self.listResults.SetColumnWidth(4, wxLIST_AUTOSIZE)
        
        #self.listResults.Set(results.getAnswer())

    def UpdateDownloadProgress(self, full = 0):
        if full:
            self.listDownloads.hashToInfoMap = {}
            self.listDownloads.numToHashMap  = {}

            self.listDownloads.DeleteAllItems()

            for key in self.main.downloads.keys():
                name = self.listDownloads.nameMapping[key]
            
                try:
                    dl = self.main.getFileDownloaded(key)
                except KeyError:
                    dl = 0

                index = self.listDownloads.InsertStringItem(self.listDownloads.GetItemCount(), "")

                self.listDownloads.hashToInfoMap[key] = (name, dl)
                self.listDownloads.numToHashMap[index] = key

                self.listDownloads.SetStringItem(index, 0, str(index + 1))
                self.listDownloads.SetStringItem(index, 1, name)
                self.listDownloads.SetStringItem(index, 2, "%i%%" % dl)

            self.listDownloads.SetColumnWidth(0, wxLIST_AUTOSIZE)
            self.listDownloads.SetColumnWidth(1, wxLIST_AUTOSIZE)
            self.listDownloads.SetColumnWidth(2, wxLIST_AUTOSIZE_USEHEADER)

        else:
            for index in xrange(0, self.listDownloads.GetItemCount()):
                hash = self.listDownloads.numToHashMap[index]

                dl = self.main.getFileDownloaded(hash)

                item = self.listDownloads.GetItem(index, 2)
                prevDL = int((item.GetText()[:-1]).strip())

                if dl != prevDL:
                    self.listDownloads.SetStringItem(index, 2, "%i%%" % dl)
        
    def UpdatePublished(self):
        items = filelib.checkIfFileExists(["Name: *", "Date: *","Category: *","CCategory: *","Desc: *","MIME: *", "HASH: *", "length: *"])

        self.listPublished.hashToInfoMap = {}
        self.listPublished.numToHashMap  = {}

        self.listPublished.DeleteAllItems()

        for item in items:
            name, date, cat, ccat, desc, mime, hash, length = item.split(";")
            
            name   = filelib.Decode(name.strip())
            date   = filelib.Decode(date.strip())
            cat    = filelib.Decode(cat.strip())
            ccat   = filelib.Decode(ccat.strip())
            desc   = filelib.Decode(desc.strip())
            mime   = filelib.Decode(mime.strip())
            hash   = filelib.Decode(hash.strip())
            length = int(length.strip())

            index = self.listPublished.InsertStringItem(self.listPublished.GetItemCount(), "")

            self.listPublished.hashToInfoMap[hash] = (name, date, cat, ccat, desc, mime, length)
            self.listPublished.numToHashMap[index] = hash

            self.listPublished.SetStringItem(index, 0, str(index + 1))
            self.listPublished.SetStringItem(index, 1, name)
            self.listPublished.SetStringItem(index, 2, date)
            self.listPublished.SetStringItem(index, 3, cat)
            self.listPublished.SetStringItem(index, 4, ToReadableUnits(length))

            self.listPublished.SetColumnWidth(0, wxLIST_AUTOSIZE)
            self.listPublished.SetColumnWidth(1, wxLIST_AUTOSIZE)
            self.listPublished.SetColumnWidth(2, wxLIST_AUTOSIZE)
            self.listPublished.SetColumnWidth(3, wxLIST_AUTOSIZE)
            self.listPublished.SetColumnWidth(4, wxLIST_AUTOSIZE)

    def OnAbout(self, event):
        #dlg = wxMessageDialog(self, "This program is a proof of concept\n"
        #                      "implementation of the multicast\n"
        #                      "based peer to peer system. It allows\n"
        #                      "simple filesharing and downloading.\n\n"
        #                      "Bugreports to: andreas.bergstrom@hiof.no",
        #                      "About Multipeer", wxOK | wxICON_INFORMATION)

        s = """Multipeer IPv6/multicast P2P publishing system V1.0

Copyright  2003 Andreas Bergstrm & Audun Vaaler,
stfold University College.

Web: http://ipv6.hiof.no/multipeer/
Mail: multipeeer@hiof.no

Bugzilla: http://ipv6.hiof.no/bugzilla/"""

        dlg = wxMessageDialog(self, s, "About Multipeer", wxOK | wxICON_INFORMATION)
                              
        dlg.ShowModal()
        dlg.Destroy()

    def AddFile(self, event):
        dlg = wxFileDialog(self, "Choose a file to publish", "./", "", "*.*", wxOPEN)
        if dlg.ShowModal() == wxID_OK:
            go_on = 1
            filename = dlg.GetFilename()
            dirname = dlg.GetDirectory()
        else:
            go_on = 0
            
        dlg.Destroy()

        if go_on:
            dlg = AddFileDialog(self, filename, dirname)
            if dlg.ShowModal() == wxID_OK:
                pass
            else:
                pass
            dlg.Destroy()
    
    def MainEvent(self, event):
        self.main.Stop()
        self.main = None
        self.Destroy()
        os._exit(0)

    #def setResults(self, result):
    #    wxMutexGuiEnter()
    #    self.listResults.Set(result)
    #    wxMutexGuiLeave()

    #def setDownloads(self):
    #    setline = []
    #    for key in self.main.udownloads.keys():
    #        setline.append(self.main.udownloads[key]["Name"] + ": " + str(self.main.udownloads[key]["Downloaded"]) + "%")
    #
    #    self.listDownloads.Set(setline)

    def Search(self, event):
        if self.searchMode == 0:
            self.StartSearching()
        else:
            self.StopSearching()

    def StartSearching(self):
        self.searchMode = 1
        self.UpdateSearchControls()

        self.listResults.DeleteAllItems()
        self.listResults.hashToInfoMap = {}
        self.listResults.numToHashMap  = {}

        self.searchStartTime = time.time()
        
        self.timerSearchProgress.Start(250, false)
            
        searchstring = self.textSearch.GetValue()
        searchform = "Name: %s\nDate: %s\nCategory: %s\nCCategory: %s\nDesc: %s\nMIME: %s\nHASH: %s\nlength: %s"
        self.sessid = self.main.sendQuery(searchform % (searchstring, "*", "*", "*", "*", "*", "*", "*"))
        #self.searcher = fetchQueryResults(self)

    def StopSearching(self):
        self.timerSearchProgress.Stop()

        self.searchMode = 0
        self.UpdateSearchControls()
        self.gaugeProgress.SetValue(0)

    def Download(self, event):
        #file = event.GetString()

        pos = self.listResults.GetNextItem(-1, state = wxLIST_STATE_SELECTED)
        index = int(self.listResults.GetItem(pos, 0).GetText()) - 1
        
        hash   = self.listResults.numToHashMap[index]
        info   = self.listResults.hashToInfoMap[hash]
        name   = info[0]

        self.listDownloads.nameMapping[hash] = name
        
        if debug:
            print "GUI: hash: %s" % hash

        self.main.getFile(self.sessid, hash)
        if debug:
            print "GUI, finished self.main.getFile()"

        self.UpdateDownloadProgress(full = 1)

        if debug:
            print "GUI: moving on"
                        
        if debug:
            "Download function finished"

    def Abort(self, event):
        if DEBUG:
            print "Aborting"

class AddFileDialog(wxDialog):
    def __init__(self, parent, filename, directory):
        wxDialog.__init__(self, parent, -1, "Publish a file")
        self.filename = filename
        self.dirname = directory
        self.parent = parent

        mime = mimetypes.guess_type(filename, strict = 0)[0]
        if not mime:
            mime = "application/octet-stream"

        catMapping = {
            "application": 0,
            "audio":       1,
            "image":       2,
            "text":        3,
            "video":       4,
            }

        mm = mime.split("/")[0]

        if mm in catMapping.keys():
            cat = catMapping[mm]
        else:
            cat = 5
        
        self.lblName = wxStaticText(self, -1, "Name:")
        self.lblCat = wxStaticText(self, -1, "Category:")
        self.lblMIME = wxStaticText(self, -1, "MIME type:")
        
        self.txtName = wxTextCtrl(self, -1, filename)
        self.txtMIME = wxTextCtrl(self, -1, mime)
        self.cmbCat = wxComboBox(self, -1, choices=["Application", "Audio", "Image", "Text", "Video", "Other"], style=wxCB_DROPDOWN)
        self.cmbCat.SetSelection(cat)
        self.boxDesc = wxStaticBox(self, -1, label = "Description")
        self.sizerDesc = wxStaticBoxSizer(self.boxDesc, wxVERTICAL)
        self.txtDesc = wxTextCtrl(self, -1, "", style=wxTE_MULTILINE | wxHSCROLL)
        self.txtDesc.SetDimensions(-1, -1, -1, 80)

        self.butOK = wxButton(self, wxOK, "&OK")
        self.butCancel = wxButton(self, wxCANCEL, "&Cancel")

        self.sizerFrame   = wxBoxSizer(wxVERTICAL)

        self.sizerTop = wxFlexGridSizer(2, 2, 2, 16)
        self.sizerDesc.Add(self.txtDesc, 1, wxEXPAND | wxALL, 4)
        self.sizerOKRow   = wxBoxSizer(wxHORIZONTAL)

        self.sizerTop.Add(self.lblName, 0, wxALL, 0)
        self.sizerTop.Add(self.txtName, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblMIME, 0, wxALL, 0)
        self.sizerTop.Add(self.txtMIME, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.Add(self.lblCat, 0, wxALL, 0)
        self.sizerTop.Add(self.cmbCat, 0, wxALL | wxEXPAND, 0)
        self.sizerTop.AddGrowableCol(1)

        self.sizerOKRow.Add(self.butOK, 0, wxALL, 4)
        self.sizerOKRow.Add(self.butCancel, 0, wxALL, 4)

        self.sizerFrame.Add(self.sizerTop, 0, wxALL | wxEXPAND, 8)
        self.sizerFrame.Add(self.sizerDesc, 1, wxALL | wxEXPAND, 4)
        self.sizerFrame.Add(self.sizerOKRow, 0, wxALL, 4)

        self.SetSizer(self.sizerFrame)
        self.sizerFrame.SetSizeHints(self)

        self.CentreOnParent(wxBOTH)

        EVT_BUTTON(self, wxOK, self.OnOK)
        EVT_BUTTON(self, wxCANCEL, self.OnCancel)

    def OnOK(self, event):

        date     = filelib.Encode(time.strftime("%Y-%m-%d", time.gmtime()))
        category = filelib.Encode(self.cmbCat.GetValue().strip())
        ccategory = ""
        description = filelib.Encode(self.txtDesc.GetValue().strip())
        mime = filelib.Encode(self.txtMIME.GetValue().strip())
        try:
            file = open(os.path.join(self.dirname, self.filename))
            fileread = file.read()
            file.close()
        except IOError:
            print "Could not read from file."
            print self.dirname, self.filename
            return
        filename = filelib.Encode(self.txtName.GetValue().strip())
        length = len(fileread)
        hashable = sha.new(fileread)
        hash = hashable.hexdigest()
        hashable = None
        description = string.replace(description, "\n", "")
        filedesc = filename + "; " + date + "; " + category + "; " + ccategory + "; " + description + "; " + mime + "; " + hash + "; " + str(length) + "\n"

        #Copy file to new location
        try:
            newfile = open(os.path.join("files", filename), "w")
        except IOError:
            print "Error could not create files/%s !" % filename

        try:
            newfile.write(fileread)
        except IOError:
            print "Error, could not write files/%s !" % filename

        try:
            newfile.close()
        except IOError:
            pass

        fileread = None
        
        #Write new file summary
        try:
            summary = open("summary.txt", "a")
        except IOError:
            print "Error could not open summary.txt (gui.py)"

        try:
            summary.write(filedesc)
        except IOError:
            print "Error could not write to file summary.txt (gui.py)"
        try:
            summary.close()
        except IOError:
            print "Error, could not close file summary.txt (gui.py)"
        self.SetReturnCode(wxOK)
        self.Close(true)
        #os.spawnlp(os.P_WAIT, 'cp', 'cp', 'index.html', '/dev/null')

        self.parent.UpdatePublished()
       
    def OnCancel(self, event):
        self.SetReturnCode(wxCANCEL)
        self.Close(true)
    
    def __set_properties(self):
        # begin wxGlade: AddFileDialog.__set_properties
        self.SetTitle("Add file description")
        self.combo_box_1.SetSelection(0)
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: AddFileDialog.__do_layout
        grid_sizer_1 = wxGridSizer(3, 6, 0, 0)
        grid_sizer_1.Add(self.Name, 0, 0, 0)
        grid_sizer_1.Add(self.Date, 0, 0, 0)
        grid_sizer_1.Add(self.Category, 0, 0, 0)
        grid_sizer_1.Add(self.CCategory, 0, 0, 0)
        grid_sizer_1.Add(self.Description, 0, 0, 0)
        grid_sizer_1.Add(self.MIME_type, 0, 0, 0)
        grid_sizer_1.Add(self.text_ctrl_1, 0, 0, 0)
        grid_sizer_1.Add(self.text_ctrl_2, 0, 0, 0)
        grid_sizer_1.Add(self.combo_box_1, 0, 0, 0)
        grid_sizer_1.Add(self.text_ctrl_3, 0, 0, 0)
        grid_sizer_1.Add(self.text_ctrl_5, 0, wxEXPAND, 0)
        grid_sizer_1.Add(self.text_ctrl_6, 0, 0, 0)
        grid_sizer_1.Add(self.button_1, 0, 0, 0)
        grid_sizer_1.Add(self.button_2, 0, 0, 0)
        self.SetAutoLayout(1)
        self.SetSizer(grid_sizer_1)
        grid_sizer_1.Fit(self)
        grid_sizer_1.SetSizeHints(self)
        self.Layout()
        # end wxGlade

    
class App(wxApp):
    def OnInit(self):
        frame = MainFrame(None, -1, "MultiPeer")
        self.SetTopWindow(frame)
        return true

#class fetchQueryResults(threading.Thread):
#    def __init__(self, parent):
#        threading.Thread.__init__(self)
#        self.parent = parent
#        self.start()
#    def run(self):
#        while(1):
#            time.sleep(2.5)
#            results = self.parent.main.getQueryResult(self.parent.sessid)
#            self.parent.setResults(results.getAnswer())

#class fetchDownloadResults(threading.Thread):
#    def __init__(self, parent):
#        threading.Thread.__init__(self)
#        self.parent = parent
#        self.start()
#    def run(self):
#        while(1):
#            time.sleep(2)
#            results = []
#            for key in self.parent.main.udownloads.keys():
#                self.parent.main.udownloads[key]["Downloaded"] = (self.parent.main.getFileDownloaded(key))
#            self.parent.setDownloads()

app = App(0)
app.MainLoop()
