#!/usr/bin/python2

"""    Multipeer, the main API for the multicast based peer to peer system, Copyright 2003 Andreas Bergstrm and Audun Vaaler

    This file is part of Multipeer.
    
    Multipeer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"""

import threading, select, time, string, network, filelib, socket, random, time
import select, copy

debug = 0 #20

def genMultipeerHeader(version, Q, L, R, E):
    """genMultipeerHeader takes five options:
    the version number, which is a to digit string,
    and true (1) and false(0) for the four flags, Q, L, R and E"""
    if Q == 0:
        Q = ""
    else:
        Q = 'Q'
    if L == 0:
        L = ""
    else:
        L = 'L'
    if R == 0:
        R = ""
    else:
        R = 'R'
    if E == 0:
        E = ""
    else:
        E = 'E'
    return "MFP2P%s%s%s%s%s:\n" % (version, Q, L, R, E)

# Parent class for all threaded classes
class Runners(threading.Thread):
    """The Runners class is an empty thread class, with a couple of
    functions added which are missing in the Python thread
    implementation"""
    def __init__(self):
        threading.Thread.__init__(self)
        self.stop = 0
        self.runners = []

    def read(self, size):
        print "Read method not implemented!"

    def write(self, message):
        print "Write method not implemented!"

    def run(self):
        print "Run method not implemented!"

    def addRunner(self, runner):
        """The addrunner function can be used to store the references
        of threads spawned by this thread, for stopping them when
        the thread itself receives a Stop()"""
        self.runners.append(runner)

    def addRunners(self, runners):
        """If there are more than one thread to add, one can use this
        function, and input a list of threads."""
        self.runners.extend(runners)

    def Stop(self):
        """The big missing function in Python threads, a stop function."""
        self.stop = 1

        for runner in self.runners:
            runner.Stop()


# A socket listener class, when it reads from socket, it sends
# the data to a doJob object.
class socketMulticastListener(Runners):
    """The socketMulticastListener class is a threaded class which
    listens to a multicast group, and sends the data read from
    that group to a worker class defined when instantiated."""
    def __init__(self, socket, worker):
        """The socket should be a multicast socket, and the worker any
        object implementing the worker class, specifically the
        work() function."""
        Runners.__init__(self)
        self.name = "socketMulticastListener (%s) : %s" % (worker.name, time.time())
        self.dataLastReceived = time.time()
        self.worker = worker
        self.socket = socket
        self.indata = ''
        self.start()

    def run(self):
        while not self.stop:
            try:
                self.indata = self.socket.recvfrom(1024)
            except:
                #Do nothing
                pass
                
            if self.indata == None:
                time.sleep(1)
            else:
                if self.indata[0].strip() == "PING":
                    if debug:
                        print "Received a multicast ping"
                    try:
                        sendsock = network.createUnicastClientSocket(self.indata[1][0], int(self.indata[1][1]))
                        #sendsock.connect((self.indata[1][0], int(self.indata[1][1])))
                        sendsock.sendall("PONG")
                        sendsock.close()
                    except IOError:
                        pass
                    
                self.worker.work(self.indata)
                if debug >= 3:
                    print "Received data over multicast: \n%s" % self.indata[0]

    def Stop(self):
        self.stop = 1
        self.socket.close()

class socketUnicastListener(Runners):
    """The socketUnicastListenerClass is a multithreaded class
    which listens to a unicast socket, and spawns a new
    thread whenever new data is ready to be read."""
    def __init__(self, parent, s, worker):
        """The parent should be a reference to the object
        instantiating the socketUnicastListener, s
        is the socket object and worker is the class
        to spawn when there is data on the socket."""
        Runners.__init__(self)
        self.name = "socketUnicastListener (%s)" % (time.time())
        self.dataLastReceived = time.time()
        self.parent = parent
        self.worker = worker
        self.s = s
        #self.addRunner(worker)
        self.start()

    def run(self):
        while not self.stop:
            check, waste, waste2 = select.select([self.s], [], [], 10)
            if check == []:
                time.sleep(1)
                pass
            else:
                for item in check:
                        work = self.worker(self.parent)
                        self.addRunner(work)
                        work.work(item)

    def Stop(self):
        self.s.close()

        for runner in self.runners:
            runner.Stop()
        
        __stop

class worker:
    """The worker class is just an empty superclass."""
    def work(self, indata):
        print "Method work not implemented!"

class superworker(Runners, worker):
    """The superworker is a worker which is threaded."""
    def __init__(self):
        Runners.__init__(self)
        
    def work(self, indata):
        print "Method work not implemented!"

class fileWorker(superworker):
    """The fileWorker handles all file sending and receving."""
    def __init__(self, parent):
        superworker.__init__(self)
        self.parent = parent
        self.start()
        self.UpLoaders = {}

    def getUpLoaders(self):
        if debug >= 1:
            print "getUpLoaders: %s" % self.UpLoaders
        return self.UpLoaders
        
    def run(self):
        while not self.stop:
            time.sleep(1)

    def stopUpload(self, hash):
        if self.UpLoaders.has_key(hash):
            self.UpLoaders[hash].stopUpLoad()
            self.UpLoaders.remove(hash)
            return 1
        else:
            return 0

    def stopDownload(self, hash):
        for runner in self.runners:
            if runner.hash == hash.strip():
                runner.stopDownload()
                self.runners.remove(runner)
                runner.Stop()
                del runner
                break

    def startDownload(self, query, hash):
        """startDownload is used to start a new subFileWorker, which
        is to download the file from the Query query with the file
        hash of hash."""
        if debug >=2:
            print "fileWorker, creating subFileWorker"
        work = subFileWorker(self)
        if debug >=2:
            print "fileWorker, starting subFileWorker with work"
        work.startDownload(query, hash)
        if debug >=2:
            print "FileWorker, adding work to runners list"
        self.addRunner(work)
        if debug >=2:
            print "filwWorker, startDownload finished."

    def startUpload(self, hash, socket):
        """startUpload is used to start a new subFileWorker, which
        is to upload a file to the peer waiting on the socket
        socket."""
        # If there already is an upload in progress for the hash
        if self.UpLoaders.has_key(hash[0:(string.find(hash, ":"))]):
            # Extract the reference to the process uploading
            work = self.UpLoaders[hash[0:(string.find(hash, ":"))]]
            # And ask it to start communicating with the new client, using the socket socket.
            if debug:
                print "FileWorker, startUpload, adding (%s) to existing upload %s" % (hash, work.getName())
            work.addClient(socket, hash)
        else:
            if debug:
                print "FileWorker, startUpload, starting upload: %s" % hash
            # Else start a new subfileworker, and start uploading the file
            work = subFileWorker(self)
            work.startUpload(hash.strip(), socket)
            self.addRunner(work)
            self.addUpLoader(hash[0:(string.find(hash, ":"))], work)

    def addUpLoader(self, hash, worker):
        self.UpLoaders[hash] = worker

class subFileWorker(superworker):
    """The subFileWorker is spawned by the FileWorker, and is the
    class which gets the file transfer done."""
    def __init__(self, parent):
        """The parent is a reference to the instantiating class."""
        superworker.__init__(self)
        self.parent = parent
        self.upload = 1
        self.download = 2
        self.donothing = 0
        self.work = self.donothing
        self.file = None
        self.random = random.Random()
        self.blockcount = 0
        self.multisock = None
        self.multisocks = []
        self.unisocks = []
        self.timeout = 0
        self.nexttime = 0
        self.start()

    def run(self):
        stringblocks = ""

        while not self.stop:
            if self.work == self.upload:
                # Get the oldest block
                if len(self.sendblocks) != 0:
                    fileseek = self.sendblocks.pop(0)
                    # Seek to correct place
                    try:
                        self.file.seek(1024 * fileseek)
                    except:
                        if debug > 5:
                            print "FileSeek: %s, %s" % (fileseek, 1024 * fileseek)
                    # Read data
                    filedata = self.file.read(1024)
                    # Generate the packet to send, and send it.
                    self.multisock.sendto((genMultipeerHeader("01", 1, 0, 1, 0) + fileseek.__str__() + ":" + self.hash.strip() + "\n" + filedata), (self.group, 2323))
                else:
                    # Let the peer know that the sendqueue is empty
                    if debug > 5:
                        print "End of Blocks!"
                    for unisock in self.unisocks:
                        try:
                            unisock.sendall("EOB")
                        except:
                            self.unisocks.remove(unisock)
                            if debug:
                                print "subFileWorker, run, upload, removing socket %s" % unisock

                insock, outsock, exceptsock = select.select(self.unisocks, [], [], 0.01)
                if len(insock) != 0:
                    readdata = None
                    for sock in insock:
                        try:
                            readdata = sock.recv(10240)
                        except:
                            pass
                        if readdata != None:
                            # Add the blocks requested to the send queue
                            if debug > 4:
                                print "Received data over unicast\n%s" % readdata
                            # Check if the client has recieved all data and
                            # add those blocks which it has not received
                            # Remove those socket to finished clients
                            if self.addBlocks(readdata) == "ALLRECV":
                                if len(self.unisocks) > 1:
                                    self.unisocks.remove(sock)
                                    sock.close()
                                else:
                                    self.unisocks.remove(sock)
                                    sock.close()
                                    self.nexttime = 1

            # To ensure smooth shutdown
            if self.nexttime == 2:
                if len(self.sendblocks) == 0:
                    self.nexttime == 0
                    self.work = self.donothing
                else:
                    pass
            elif self.nexttime == 1:
                self.nexttime = 2
                    
            if self.work == self.download:
                if debug > 10:
                    print "Len stringblocks: %s" % (len(stringblocks))
                for sock in self.multisock:
                    
                    try:
                        data = sock.recvfrom(1500)
                    except socket.error, msg:
                        if debug:
                            print "subFileWorker, run, download, error %s (%s) on insock" % (socket.error, msg)
                            break
                    # Get block number and data
                    block, parseddata = self.parse(data)
                    
                    #Seek to correct place on disk
                    if debug >= 10:
                        print "subFileWorker, now getting block %s" % block
                    if block != None:
                        self.file.seek(block * 1024)
                        # Write to disk
                        self.file.write(parseddata)
                        # Flush write buffer to disk
                        # self.file.flush()
                        # Mark block as received
                        try:
                            self.recblocks.remove(block)
                            # Mark another block downloaded
                            self.parent.parent.setFileDownloaded(self.filequery["HASH"])
                        except ValueError:
                            pass
                    
                    # TODO: need to thread this, it slows down downloading speed
                insock, outsock, exceptsock = select.select(self.unisock, [], [], 0.01)

                if len(insock) != 0:
                    if debug:
                        print "subFileWorker, run, download, unicast received."
                    for socket in insock:
                        try:
                            readdata = socket.recv(2048)
                        except:
                            break
                        if readdata != None:
                            # If end of blocks is received from peer
                            if readdata.strip() == "EOB":
                                self.file.flush()
                                if debug:
                                    print "EOB received!"
                                # If all blocks are received
                                if len(self.recblocks) == 0:
                                    if debug:
                                        print "All blocks has been reveived"
                                    # Let the peer know that all blocks
                                    # have been received
                                    socket.sendall("ALLRECV")
                                    if debug:
                                        print "subFileWorker, All Received"
                                    self.file.flush()
                                    # Close the socket
                                    socket.close()
                                    self.unisock.remove(socket)
                                    # Stop self
                                    if len(self.unisock) == 0:
                                        self.work = self.donothing
                                else:
                                    # Else send which blocks are missing.
                                    if debug:
                                        print "Asking for more blocks: %s, %s" % (len(self.recblocks), len(stringblocks))
                                    
                                    if len(stringblocks) < 10:
                                        if debug > 10:
                                            print "subFileWorker, generating stringblocks!"
                                        for block in self.recblocks:
                                            if len(stringblocks) == 0:
                                                stringblocks += str(block)
                                            else:
                                                stringblocks += "," + str(block)
                                                
                                        strings = len(stringblocks)
                                        if strings > 50:
                                            stringsendblocks = stringblocks[:(strings/self.hostdownloads)]
                                            socket.send(stringsendblocks)
                                        else:
                                            socket.send(stringblocks)

                                        stringblocks = stringblocks[(strings/self.hostdownloads):]
                                         
                                    else:
                                        if debug > 10:
                                            print "subFileWorker, just using stringblocks!"

                                        strings = len(stringblocks)
                                        if strings > 50:
                                            stringsendblocks = stringblocks[:(strings/self.hostdownloads)]
                                            socket.send(stringsendblocks)
                                        else:
                                            socket.send(stringblocks)

                                        stringblocks = stringblocks[(strings/self.hostdownloads):]
                                        
            # Else if work has net yet been allocated
            if self.work == self.donothing:
                # Sleep for a second
                time.sleep(1)

    def stopUpload(self):
        self.work == self.donothing
        for socket in self.unisock:
            try:
                socket.sendall("SHUTDOWN")
                socket.close()
            except:
                print "Error, could not shutdown socket"
                del socket
                
        for sock in self.multisock:
            try:
                socket.close()
            except:
                del socket

        del self.unisock
        del self.multisock
        self.Stop()

        return 1

    def startUpload(self, hash, socket):
        """startUpload initializes an upload to the peer on the other
        side of the socket socket. It fetches the file with the
        hash of hash, and starts the upload."""

        # Separate the filedescripter from the filehahs
        self.hash = hash[0:(string.find(hash, ":"))]
        self.filedesck = hash[((string.find(hash, ":")) + 1):]
        # Get name of file and filelength
        replyfile, filelen = filelib.getFile(self.hash)
        # If file does not exist
        if replyfile == None:
            # The file does not exist
            socket.sendall("NOSUCHFILE")
            # Close socket
            socket.close()
            # Stop self
            self.Stop()

        self.filelen = int(filelen)

        # Initialize the list over blocks which are to be sent.
        self.sendblocks = []

        # Add the requested blocks
        self.addBlocks(self.filedesck)

        self.file = replyfile

        # Generate random multicast group
        address= [hex(int(self.random.random() * 65537))[2:], hex(int(self.random.random() * 65537))[2:], hex(int(self.random.random() * 65537))[2:]]
        self.group = "ff3e:30:2001:700:a00:" + address[0] + ":" + address[1] + ":" + address[2]
        # Create the multicast socket
        self.multisock = network.createMulticastSocket(self.group, "2323")
        if debug:
            print "Appending socket: [%s]" % socket
        self.unisocks.append(socket)

        # Send filehash and randomly generated multicastgroup
        socket.sendall(genMultipeerHeader("01", 1, 0, 1, 0) + self.hash + "\n" + "\t.\t[" + self.group + "]:2323\t:\t")

        # Sleep for two seconds to allow the other peer to establish a
        # multicast connection
        time.sleep(0.1)
        
        # Tell run() to start uploading
        self.work = self.upload


    def addClient(self, socket, hash):

        #Extract the filedescripter from the hash
        self.filedesck = hash[((string.find(hash, ":")) + 1):]

        # Add the requested blocks from the filedescripter
        self.addBlocks(self.filedesck.strip())

        # Send filehash and randomly generated multicastgroup
        socket.sendall(genMultipeerHeader("01", 1, 0, 1, 0) + self.hash + "\n" + "\t.\t[" + self.group + "]:2323\t:\t")

        # Add the socket to the list over sockets that need to be watched
        if debug:
            print "Adding client socket [%s]" % socket
        self.unisocks.append(socket)

        if self.work == self.donothing:
            self.work = self.upload

    def stopDownload(self):
        self.work = self.donothing
        del self.hash
        return 1

    def startDownload(self, query, hash):
        """startDownload initializes a download using the hosts contained
        in the Query query for the file with a hash of hash."""

        self.hash = hash.strip()
        self.filequery = ""
        for answer in query.answer:
            if answer["HASH"].strip() == hash.strip():
                self.filequery = answer

        path = "downloads/" + self.filequery["Name"].strip()

        #Initialize file object
        file = None

        # Open file
        try:
            file = open(path, "w")
        except:
            print "Error, could not open %s for writing!" % path
            self.Stop()

        # Reserve space for the file on disk
        # Go to what will be EOF minus one
        file.seek((int(self.filequery["length"]) - 1))
        try:
            # Write the letter E
            file.write('E')
            # Flush write buffer to disk
            file.flush()
        except:
            # Should it fail, then there is probably not enough space left
            # on the disk
            # TODO: report this error in the GUI
            print "Error, probably not enough space left on device for %s" % path

        if debug >=2:
            print "subFileWorker, reserved space on disk"

        # Return to start of file
        file.seek(0)

        # Here we need to start downloading from several servers
        # Initial algorithm, the first x computers to reply

        downloads = 5

        host = self.filequery["host"]

        if len(host) < 5:
            downloads = len(host) - 1
            self.hostdownloads = len(host)

        self.unisock = []
        self.multisock = []

        if debug >=2:
            print "subFileWorker, initializing list of blocks to receive"

        # Initialize list of blocks to receive
        self.filelen = int(self.filequery["length"])
        self.recblocks = []
        for i in xrange(0, (self.filelen / 1024)):
            self.recblocks.append(i)

        # Again, if there are leftovers from the division, add one more block
        if (self.filelen % 1024) != 0:
            self.recblocks.append(i + 1)

        self.maxblock = i + 1

        self.file = file

        if debug:
            print host
            for i in range(0, self.hostdownloads):
                print host[i]

        if debug >=2:
            print "subFileWorker, connecting to all hosts."
        
        for i in range(0, self.hostdownloads):
            # Calculate which parts of the file we want
            # TODO: Interlacing?
            bottom = int((self.maxblock / (self.hostdownloads)) * i)
            top = int((self.maxblock / (self.hostdownloads)) * (i+1))
            
            if top > self.maxblock:
                top = self.maxblock
                if debug >=2:
                    print "Top is greater than self.maxblock"

            blockswanted = "%s-%s" % (bottom, top)
            #Connect to other host, ask for file

            if debug:
                print "subFileWorker, run, download, host: %s" % host[i]
            
            self.unisock.append(network.createUnicastClientSocket(host[i], 2323))
            self.unisock[i].sendall(genMultipeerHeader("01", 1, 0, 0, 0) + self.filequery["HASH"] + ":" + blockswanted + "\n\t.\t\t:\t")

            if debug >=2:
                print "Send number of blocks wanted to peer."
            
            readdata = ""
            while string.find(readdata, "\t:\t") == -1:
                try:
                    incoming, waste, waste2 = select.select([self.unisock[i]], [], [], 5)
                    if incoming == []:
                        self.unisock[i] = None
                        break
                    readdata = readdata + self.unisock[i].recv(1024)
                except:
                    print "Error while reading from socket, fileWorkerDownload!"
            if readdata == "":
                break

            if debug >=2:
                print "Received reply from peer."

            header = readdata[0:9]

            # Check that reply comes from a peer with MFP2P version 01
            if header != "MFP2P01QR":
                print "Error, not MFP2P header: %s" % header

            # Extract hash from reply
            hash = readdata[(string.find(readdata, "\n")+1):string.find(readdata, "\t.\t")]
            # If the correct hash is not in the reply, stop download
            # TODO: report this to the GUI
            if hash.strip() != self.filequery["HASH"].strip():
                print "Error did not reply with same HASH: %s" % hash
                self.unisock[i].close()
                self.unisock[i] = None

            # OK we have received a proper reply, prepare to download.

            # Extract group and port to download from
            group = readdata[(string.find(readdata, "[") + 1):string.find(readdata, "]")]
            port = readdata[(string.find(readdata, "]:") + 2):string.find(readdata, "\t:\t")]

            # Create a multicast socket
            self.multisock.append(network.createMulticastSocket(group, port))

        # Add the download to the list of downloads underway
        self.parent.parent.downloads[hash.strip()] = copy.deepcopy(query)

        if debug >=2:
            print "subFileWorker, startDownload finished."

        for a in xrange(0,5):
            try:
                self.unisock.remove(None)
            except:
                pass
        
        #Let run() know it is time to download
        self.work = self.download

    def parse(self, data):
        """Parse is helper function which parses filetransfer MFP2P
        packets."""
        #We only need first part of tuple
        data = data[0]
        head = data[0:4]
        version = data[5:7]
        Q = data[7]
        R = data[8]
        block = data[11:string.find(data, ":", 11)]
        hash = data[(string.find(data,":", 10) + 1):string.find(data, "\n", 20)]
        if not hash.strip() == self.filequery["HASH"].strip():
            print "Error, file does not have correct HASH!"
            return None, None
        if not (Q == 'Q' and R == 'R'):
            print "Not QR %s%s" % (Q, R)
            return None, None

        #Extract the data from the packet:
        data = data[(string.find(data, "\n", 20) + 1):]
        if len(data) != 1024:
            #print "Error, data not 1024 bytes! %s" % len(data)
            pass

        # Return the blocknumber and the filedata
        return int(block), data

    def addBlocks(self, blocks):
        """addBlocks is a helper function used to parse additional
        block requests, and add those blocks to the sendqueue:"""

        if blocks.strip() == "0":
            self.addAllBlocks()
            return
        
        blocks = string.split(blocks, ",")
        for block in blocks:
            if string.find(block, "-") != -1:
                if debug:
                    print block
                parts = string.split(block, "-")
                if debug:
                    print "Parts: %s" % parts
                start = int(parts[0])
                end = int(parts[1])
                if start == 0 and end == 0:
                    self.addAllBlocks()
                    return
                for i in range(start, end):
                    self.sendblocks.append(i)
            else:
                # TODO: Must change
                if block == "ALLRECV":
                    return "ALLRECV"
                elif block != "":
                    try:
                        self.sendblocks.append(int(block))
                    except ValueError:
                        if debug:
                            print "Could not convert %s to int!" % block
    def addAllBlocks(self):
        for i in range(0, (self.filelen / 1024)):
            self.sendblocks.append(i)

        # If there is a leftover from the division, and one more block
        if (self.filelen % 1024) != 0:
            self.sendblocks.append(i + 1)

class unicastWorker(superworker):
    """The unicastworker handles the actucal work when a
    unicastListener notices that there is data to read."""
    def __init__(self, parent):
        """Parent is the instantiating object."""
        self.parent = parent
        superworker.__init__(self)
        self.name = "unicastWorker %s" % (time.time())
        self.start()
        
    def work(self, socket):
        """work takes one argument, the socket for which there are
        data waiting."""
        if debug:
            print "Receiving unicast transmission"
        # Create a new socket from the socket with inndata
        worksock = socket.accept()
        self.socket = worksock[0]
        self.peer = worksock[1]

        readdata = ""
        # Until we reach end of packet, loop
        while string.find(readdata, "\t:\t") == -1 and string.find(readdata, "PING") == -1 and string.find(readdata, "PONG") == -1:
            if debug > 15:
                print "Start pingloop"
            try:
                read = self.socket.recv(1024)
                readdata += read
            except IOError:
                print "Error while reading from socket, unicastWorker!"
                time.sleep(0.01)

            if debug > 15:
                print "End of pingloop"

        if debug:
            print "Analyzing unicast data"

        if string.find(readdata, "PING") != -1:
            self.socket.sendall("PONG")
            if debug:
                print "Receiced a ping!"
            self.socket.close()
            return

        if string.find(readdata, "PONG") != -1:
            if debug:
                print "Received a pong!"
            self.parent.pong(self.peer[0])
            self.socket.close()
            return

        # Creating proper parseMP2P format
        readdata = [readdata, [self.peer[0]]]

        # Parse the packet.
        parseddata = parseMP2P(readdata)

        # Somehow, very unlikely, we got more than one request.
        if len(parseddata) != 1:
            print "Error, unicastWorker,  more than one request!"
            return None
        
        data = parseddata[0]
        
        # Process the data, and either send the result of a query
        # on the the parent thread, or send a download request on
        # to the parent thread which will send it on to the
        # fileWorker
        if data.version == "01":
            if data.L and data.R:
                if debug:
                    print "Reply to a locator packet"
                self.parent.passQueryResult(data)
                self.socket.close()
                self.Stop()
            
            elif data.Q:
                if debug:
                    print "A file query received"
                self.parent.sendFile(data.query, self.socket)

            else:
                print "Error, unicastWorker, did not receive a reply or wrong version."
                self.socket.close()
                self.Stop()

    def run(self):
        # Just keep looping and keeping the object alive
        while not self.stop:
            time.sleep(1)

class MP2P:
    """Class MP2P, which really should be called MFP2P is just
    a class containing all the parts of proper MFP2P packets."""
    def __init__(self, version, host, Q, L, R, E, query, group, port):
        self.host = host
        self.version = version
        self.Q = Q
        self.L = L
        self.R = R
        self.E = E
        self.query = query
        self.group = group
        self.port = port
        self.session = None

def parseMP2P(data):
    """parses a string, normally from a socket, into a MP2P object."""
    returnlist = []
    
    if len(data) == 2:
        header = data[0]
        version = header[5:7]

        #Fetch the flags
        flags = header[6:(string.find(header, ':'))]

        if 'Q' in flags:
            Q = 1
        else:
            Q = 0
        if 'L' in flags:
            L = 1
        else:
            L = 0
        if 'R' in flags:
            R = 1
        else:
            R = 0
        if 'E' in flags:
            E = 1
        else:
            E = 0

        query = header[(string.find(header, ":") + 1):(string.find(header, "\t.\t"))]
            
        if string.find(header, "[") and string.find(header, "]"):
            group = header[(string.find(header, "[") + 1):(string.find(header, "]"))]
            port = header[(string.find(header, "]") + 2):]
        else:
            group = None
            port = None
        
        newMP2P = MP2P(version, data[1][0], Q, L, R, E, query, group, port)
        returnlist.append(newMP2P)
    elif len(data) != 0:
        print "Error in parsing, got more than one tuppel from recvfrom!"

    return returnlist
        
    

class queryAnsweringMachine(worker):
    """the queryAnsweringMachine is the worker class which parses
    the multicast locator queries."""
    def __init__(self):
        self.name = "queryAnsweringMachine %s" % (time.time())
        
    def work(self, indata):
                   
        packets = parseMP2P(indata)

        for packet in packets:
            if packet.version == '01':
                if packet.L and not packet.R:
                    dosomthing = queryLocatorHandler(packet)
                else:
                    print "Error malformed multicast packet!"

def queryLocatorHandler(packet):
    """The queryLocatorHandler takes the data read from a socket
    in queryAnsweringMachine, parses it, checks if it finds
    what it is asking for, and replies by unicast."""
    try:
        replys = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        replys.connect((packet.host, 2323))
    except:
        print "queryLocatorHandler, could not open socket to answer a query"
        return
    
    reply = filelib.checkIfFileExists(string.split(packet.query[1:], "\n"))
    data = ""
    for line in reply:
        retdata = ""
        for part in line:
            retdata += part
        data += retdata
    header = genMultipeerHeader("01", 0, 1, 1, 0)
    data = header + data + "\t.\t[" + packet.group + "]:0\n\t:\t"
    replys.sendall(data)
    if debug:
        print "queryLocatorHandler, reply sent \n %s" % data
    replys.close()
    return None

class Query:
    """The Query class contains all information related to a query, such
    as the sessionID, the answers received and the original query, as
    well as download information."""
    def __init__(self, sessionID, query):
        "Query is here a string, the orignal query."
        self.sessionID = sessionID
        self.query = query
        self.answer = []
        self.downloaded = 0
        self.numblocks = 0

    def getSessionID(self):
        "Returns the sessionID of the Query"
        return self.sessionID

    def getQuery(self):
        "Returns the original query"
        return self.query

    def getAnswer_old(self):
        retstring = []
        
        for line in self.answer:
            retstring.append("%s, %s,%s, %s, %s, %s, %s, %s" % (line["Name"], line["Date"], line["Category"], line["CCategory"], line["Desc"], line["MIME"], line["HASH"], line["length"]))

        return retstring

    def getAnswer(self):
        res = []
        
        for line in self.answer:
            #res.append((line["Name"], line["Date"], line["Category"], line["CCategory"], line["Desc"], line["MIME"], line["HASH"], line["length"]))
            res.append(line)

        return res

    def insertQueryLine(self, qline):
        "qline is a reply to the original query, a string."
        # def queryLine: host, summary.txt info
        found = 0
        for line in self.answer:
            if qline["HASH"] == line["HASH"]:
                line["host"].append(qline["host"][0])
                found = 1

        if not found:
            self.answer.append(qline)
                    
    def setDownloaded(self, hash):
        """Increase the download status of the file with hash hash
        with one block."""
        if self.downloaded == 0:
            for line in self.answer:
                if line["HASH"].strip() == hash.strip():
                    length = int(line["length"].strip())
            self.numblocks = length / 1024
            if (length % 1024) != 0:
                self.numblocks += 1
            
        self.downloaded += 1

    def getDownloaded(self):
        if debug >=2:
            print "getDownloaded in query activated!"
        if self.numblocks != 0:
            if debug >=2:
                print "self.downloaded: %s, self.numblocks %s" % (self.downloaded, self.numblocks)
            return ((float(self.downloaded) * 1.0) / (float(self.numblocks) * 1.0)) * 100.0
        else:
            if debug >=2:
                print "self.numblocks are zero"
            return 0

class Main(threading.Thread):
    """Class main is the API and the only class an UI designer should
    interface against, the interesting commands are:
    sendQuery(self, query), which is used to send a query, it returns the
                            sessionID
    getQueryResult(self, sessionID), which returns the answers to the query
    getFile(self, query, hash), which starts a download of the the file
    hash hash from the Query query
    getFileDownloaded(self, hash), which returns the download status of
                                   the file with the hash hash
    """
    def __init__(self):
        threading.Thread.__init__(self)
        self.stop = 0
        self.queries = {}
        self.downloads = {}
        self.udownloads = {}
        self.pongers = {}
        
        threads = []
        self.sessions = 0

        testsocket = network.createMulticastSocket("ff3e:30:2001:700:a00:5::1", "2323", 45) 
        usocket = network.createUnicastSocket(None, "2323")

        worker = queryAnsweringMachine()
        
        self.superworker = fileWorker(self)
        threads.append(superworker)
        
        uworker = unicastWorker

        mlistener = socketMulticastListener(testsocket, worker)
        threads.append(mlistener)

        ulistener = socketUnicastListener(self, usocket, uworker)
        threads.append(ulistener)
        self.threads = threads
        self.start()

    def Stop(self):
        self.stop = 1
    
    def run(self):
        while not self.stop:
            time.sleep(1)

        for thred in self.threads:
            thred.Stop()

        self.Stop()

    def ping(self):
        """Send a 'Ping' out to see which other multipeers are currently
        online"""
        s = network.createMulticastSocket("ff3e:30:2001:700:a00:5::1", 2323)
        s.sendto("PING", ("ff3e:30:2001:700:a00:5::1", 2323))
        s.close()

    def getPongers(self):
        return self.pongers

    def pong(self, host):
        """command which is used to register an answer to a ping"""
        if self.pongers.has_key(host):
            self.pongers[host] += 1
        else:
            self.pongers[host] = 1

    def clearPong(self):
        """Clear list of ping replies"""
        self.pongers = {}
        
    def newSessionID(self):
        """Returns a new sessionID"""
        former = self.sessions
        new = former + 1
        self.sessions = new
        return new.__str__()
            
    def sendQuery(self, query):
        """Takes a properly formated query, packs in into a MFP2P packet,
        and sends it, returning the sessionID for later use."""
        sessionID = self.newSessionID()
        s = network.createMulticastSocket("ff3e:30:2001:700:a00:5::1", 2323)
        s.sendto((genMultipeerHeader("01", 0, 1, 0, 0) + query + "\n\t.\t[" + sessionID + "]:0\t:\t"), ("ff3e:30:2001:700:a00:5::1", 2323))
        s.close()
        
        newquery = Query(sessionID, query)
        self.queries[sessionID] = newquery
        return sessionID

    def passQueryResult(self, MP2P):
        """Passes the query result from unicastListener to this module."""
        sessionID = MP2P.group
        result = MP2P.query
        # Transfer a query result from unicastListener to this module
        splitresult = string.split(result, "\n")
        for line in splitresult:
            # If empty line, break
            if line != "":
                data = string.split(line, ";")
                query = {"Name" : filelib.Decode(data[0]), "Date" : filelib.Decode(data[1]), "Category" : filelib.Decode(data[2]), "CCategory" : filelib.Decode(data[3]), "Desc" : filelib.Decode(data[4]), "MIME" : filelib.Decode(data[5]), "HASH" : filelib.Decode(data[6]), "length" : data[7], "host" : [MP2P.host]}

                if debug:
                    print "Keys2: self.queries.keys(): %s" % self.queries.keys()
                    print "KeyExists: %s" % self.queries.has_key(sessionID)
                
                self.queries[sessionID].insertQueryLine(query)

    def getUpLoads(self):
        "Returns a dictionary over UpLoads in progress."
        if debug >=1:
            ret = self.superworker.getUpLoaders()
            print "getUpLoads: %s" % ret
            return ret
        else:
            return self.superworker.getUpLoaders()

    def getUpLoadsList(self):
        "Returns a list with full metadata over UpLoads in progress."
        keys = self.getUpLoads().keys()
        if debug >= 1:
            print "getUpLoadsList, keys: %s" % keys
        returns = []
        for key in keys:
            if debug >= 1:
                print "Key is: %s" % key
                print key
            ret = filelib.checkIfFileExists(["Name: *","Date: *","Category: *","CCategory: *","Desc: *","MIME: *", ("HASH: " + key.strip()), "length: *"])
            if debug >= 1:
                print ret
            returns.append(ret)

        return returns

        
    def getQueryResult(self, sessionID):
        "Returns the result of a query as a Query object"
        return self.queries[sessionID]

    def sendFile(self, hash, socket):
        "Pass a file download query from the unicastListener to the fileworker"
        self.superworker.startUpload(hash.strip(), socket)

    def getFile(self, sessionID, hash):
        """Start a file download, use the servers from sessionID which had the
        file with a hash of hash"""
        if debug >=2:
            print "Main.getFile starting"
        self.superworker.startDownload(self.queries[sessionID], hash.strip())
        if debug >=2:
            print "Main.getFile finished"

    def removeFileFromSharing(self, hash):
        """Stop the download with the hash of hash and remove it from
        summary text"""
        self.stopUpload(hash)
        filelib.removeFromSharing(hash)

    def stopUpload(self, hash):
        "Stop the upload with the hash of hash"
        self.superworker.stopUpload(hash)

    def stopDownload(self, hash):
        "Stop the download with the hash of hash"
        self.superworker.stopDownload(hash)

    def setFileDownloaded(self, hash):
        """Increase the amount of downloaded blocs by a value of one"""
        try:
            self.downloads[hash.strip()].setDownloaded(hash.strip())
        except:
            print "SetFileDownloaded, error: %s" % hash
    def getFileDownloaded(self, hash):
        "Get the percentage of file downloaded"
        if debug >=2:
            print "Main.getFileDownloaded started"

        print self.downloads[hash.strip()]
            
        return self.downloads[hash.strip()].getDownloaded()
